@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -149,8 +149,9 @@ static void at_csca_query_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	if (number[0] == '+') {
 		number = number + 1;
 		sca.type = 145;
-	} else
+	} else {
 		sca.type = 129;
+	}
 
 	strncpy(sca.number, number, OFONO_MAX_PHONE_NUMBER_LENGTH);
 	sca.number[OFONO_MAX_PHONE_NUMBER_LENGTH] = '\0';
@@ -329,7 +330,8 @@ static void at_cmt_notify(GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	ofono_debug("Got new SMS Deliver PDU via CMT: %s, %d", hexpdu, tpdu_len);
+	ofono_debug("Got new SMS Deliver PDU via CMT: %s, %d",
+			hexpdu, tpdu_len);
 
 	decode_hex_own_buf(hexpdu, -1, &pdu_len, 0, pdu);
 	ofono_sms_deliver_notify(sms, pdu, pdu_len, tpdu_len);
