@@ -38,40 +38,17 @@
 #include <ofono/log.h>
 #include <ofono/modem.h>
 #include <ofono/sim.h>
-
-#include "isi.h"
 #include "simutil.h"
 
-#define PN_SIM			0x09
-#define SIM_TIMEOUT		5
-#define SIM_MAX_IMSI_LENGTH	15
-
-enum return_code {
-	SIM_SERV_OK = 0x01,
-};
-
-enum message_id {
-	SIM_IMSI_REQ_READ_IMSI = 0x1D,
-	SIM_IMSI_RESP_READ_IMSI = 0x1E,
-	SIM_SERV_PROV_NAME_REQ = 0x21,
-	SIM_SERV_PROV_NAME_RESP = 0x22
-};
-
-enum service_types {
-	SIM_ST_READ_SERV_PROV_NAME = 0x2C,
-	READ_IMSI = 0x2D,
-};
+#include "isimodem.h"
+#include "isiutil.h"
+#include "sim.h"
+#include "debug.h"
 
 struct sim_data {
 	GIsiClient *client;
 };
 
-static void sim_debug(const void *restrict buf, size_t len, void *data)
-{
-	DBG("");
-	dump_msg(buf, len);
-}
-
 /* Returns fake (static) file info for EFSPN */
 static gboolean efspn_file_info(gpointer user)
 {
@@ -96,11 +73,11 @@ static void isi_read_file_info(struct ofono_sim *sim, int fileid,
 		return;
 	}
 
-	DBG("Not implemented (fileid = %04x)",fileid);
+	DBG("Not implemented (fileid = %04x)", fileid);
 	CALLBACK_WITH_FAILURE(cb, -1, -1, -1, NULL, data);
 }
 
-static bool spn_resp_cb(GIsiClient * client, const void *restrict data,
+static bool spn_resp_cb(GIsiClient *client, const void *restrict data,
 			size_t len, uint16_t object, void *opaque)
 {
 	const unsigned char *msg = data;
@@ -173,7 +150,7 @@ error:
 	if (cbd)
 		g_free(cbd);
 
-	DBG("Not implemented (fileid = %04x)",fileid);
+	DBG("Not implemented (fileid = %04x)", fileid);
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, data);
 }
 
@@ -181,7 +158,7 @@ static void isi_read_file_linear(struct ofono_sim *sim, int fileid,
 					int record, int length,
 					ofono_sim_read_cb_t cb, void *data)
 {
-	DBG("Not implemented (fileid = %04x)",fileid);
+	DBG("Not implemented (fileid = %04x)", fileid);
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, data);
 }
 
@@ -189,7 +166,7 @@ static void isi_read_file_cyclic(struct ofono_sim *sim, int fileid,
 					int record, int length,
 					ofono_sim_read_cb_t cb, void *data)
 {
-	DBG("Not implemented (fileid = %04x)",fileid);
+	DBG("Not implemented (fileid = %04x)", fileid);
 	CALLBACK_WITH_FAILURE(cb, NULL, 0, data);
 }
 
@@ -198,7 +175,7 @@ static void isi_write_file_transparent(struct ofono_sim *sim, int fileid,
 					const unsigned char *value,
 					ofono_sim_write_cb_t cb, void *data)
 {
-	DBG("Not implemented (fileid = %04x)",fileid);
+	DBG("Not implemented (fileid = %04x)", fileid);
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
@@ -207,7 +184,7 @@ static void isi_write_file_linear(struct ofono_sim *sim, int fileid,
 					const unsigned char *value,
 					ofono_sim_write_cb_t cb, void *data)
 {
-	DBG("Not implemented (fileid = %04x)",fileid);
+	DBG("Not implemented (fileid = %04x)", fileid);
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
@@ -215,7 +192,7 @@ static void isi_write_file_cyclic(struct ofono_sim *sim, int fileid,
 					int length, const unsigned char *value,
 					ofono_sim_write_cb_t cb, void *data)
 {
-	DBG("Not implemented (fileid = %04x)",fileid);
+	DBG("Not implemented (fileid = %04x)", fileid);
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
@@ -231,7 +208,7 @@ static bool imsi_resp_cb(GIsiClient *client, const void *restrict data,
 	size_t j = 0;
 	size_t octets = 0;
 
-	if(!msg) {
+	if (!msg) {
 		DBG("ISI client error: %d", g_isi_client_error(client));
 		goto error;
 	}
@@ -245,7 +222,7 @@ static bool imsi_resp_cb(GIsiClient *client, const void *restrict data,
 	octets = msg[3];
 	if (octets != 8 || octets > len)
 		goto error;
-	
+
 	msg += 4;
 
 	/* Ignore the low-order semi-octet of the first byte */
@@ -286,7 +263,7 @@ static void isi_read_imsi(struct ofono_sim *sim,
 	if (!cbd)
 		goto error;
 
-	if (g_isi_request_make(sd->client, msg, sizeof(msg), 
+	if (g_isi_request_make(sd->client, msg, sizeof(msg),
 				SIM_TIMEOUT,
 				imsi_resp_cb, cbd))
 		return;
