@@ -2,7 +2,7 @@
  *
  *  AT chat library with GLib integration
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -27,6 +27,7 @@ extern "C" {
 #endif
 
 #include "gatresult.h"
+#include "gatsyntax.h"
 
 struct _GAtChat;
 
@@ -36,24 +37,29 @@ typedef void (*GAtResultFunc)(gboolean success, GAtResult *result,
 				gpointer user_data);
 typedef void (*GAtNotifyFunc)(GAtResult *result, gpointer user_data);
 typedef void (*GAtDisconnectFunc)(gpointer user_data);
+typedef void (*GAtDebugFunc)(const char *str, gpointer user_data);
 
-enum _GAtChatFlags {
-	G_AT_CHAT_FLAG_NO_LEADING_CRLF = 1,	/* Some emulators are broken */
-};
+GAtChat *g_at_chat_new(GIOChannel *channel, GAtSyntax *syntax);
 
-typedef enum _GAtChatFlags GAtChatFlags;
-
-GAtChat *g_at_chat_new(GIOChannel *channel, int flags);
-GAtChat *g_at_chat_new_from_tty(const char *device, int flags);
+GIOChannel *g_at_chat_get_channel(GAtChat *chat);
 
 GAtChat *g_at_chat_ref(GAtChat *chat);
 void g_at_chat_unref(GAtChat *chat);
 
 gboolean g_at_chat_shutdown(GAtChat *chat);
 
+gboolean g_at_chat_set_syntax(GAtChat *chat, GAtSyntax *syntax);
+
 gboolean g_at_chat_set_disconnect_function(GAtChat *chat,
 			GAtDisconnectFunc disconnect, gpointer user_data);
 
+/*!
+ * If the function is not NULL, then on every read/write from the GIOChannel
+ * provided to GAtChat the logging function will be called with the
+ * input/output string and user data
+ */
+gboolean g_at_chat_set_debug(GAtChat *chat, GAtDebugFunc func, gpointer user);
+
 /*!
  * Queue an AT command for execution.  The command contents are given
  * in cmd.  Once the command executes, the callback function given by
@@ -92,24 +98,41 @@ guint g_at_chat_send(GAtChat *chat, const char *cmd,
  * intermediate responses immediately through the GAtNotifyFunc callback.
  * The final response will still be sent to GAtResultFunc callback.  The
  * final GAtResult will not contain any lines from the intermediate responses.
- * This is useful for listing commands such as CMGL or CPBR.
+ * This is useful for listing commands such as CPBR.
  */
 guint g_at_chat_send_listing(GAtChat *chat, const char *cmd,
 				const char **valid_resp,
 				GAtNotifyFunc listing, GAtResultFunc func,
 				gpointer user_data, GDestroyNotify notify);
 
+/*!
+ * Same as g_at_chat_send_listing except every response line in valid_resp
+ * is expected to be followed by a PDU.  The listing function will be called
+ * with the intermediate response and the following PDU line.
+ *
+ * This is useful for PDU listing commands like the +CMGL
+ */
+guint g_at_chat_send_pdu_listing(GAtChat *chat, const char *cmd,
+				const char **valid_resp,
+				GAtNotifyFunc listing, GAtResultFunc func,
+				gpointer user_data, GDestroyNotify notify);
+
 gboolean g_at_chat_cancel(GAtChat *chat, guint id);
+gboolean g_at_chat_cancel_all(GAtChat *chat);
 
 guint g_at_chat_register(GAtChat *chat, const char *prefix,
 				GAtNotifyFunc func, gboolean expect_pdu,
 				gpointer user_data, GDestroyNotify notify);
 
 gboolean g_at_chat_unregister(GAtChat *chat, guint id);
+gboolean g_at_chat_unregister_all(GAtChat *chat);
 
 gboolean g_at_chat_set_wakeup_command(GAtChat *chat, const char *cmd,
 					guint timeout, guint msec);
 
+void g_at_chat_add_terminator(GAtChat *chat, char *terminator,
+				int len, gboolean success);
+
 
 #ifdef __cplusplus
 }
