@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -37,6 +37,40 @@ enum ofono_sim_file_structure {
 	OFONO_SIM_FILE_STRUCTURE_CYCLIC = 3
 };
 
+enum ofono_sim_password_type {
+	OFONO_SIM_PASSWORD_NONE = 0,
+	OFONO_SIM_PASSWORD_SIM_PIN,
+	OFONO_SIM_PASSWORD_PHSIM_PIN,
+	OFONO_SIM_PASSWORD_PHFSIM_PIN,
+	OFONO_SIM_PASSWORD_SIM_PIN2,
+	OFONO_SIM_PASSWORD_PHNET_PIN,
+	OFONO_SIM_PASSWORD_PHNETSUB_PIN,
+	OFONO_SIM_PASSWORD_PHSP_PIN,
+	OFONO_SIM_PASSWORD_PHCORP_PIN,
+	OFONO_SIM_PASSWORD_SIM_PUK,
+	OFONO_SIM_PASSWORD_PHFSIM_PUK,
+	OFONO_SIM_PASSWORD_SIM_PUK2,
+	OFONO_SIM_PASSWORD_PHNET_PUK,
+	OFONO_SIM_PASSWORD_PHNETSUB_PUK,
+	OFONO_SIM_PASSWORD_PHSP_PUK,
+	OFONO_SIM_PASSWORD_PHCORP_PUK,
+	OFONO_SIM_PASSWORD_INVALID,
+};
+
+enum ofono_sim_phase {
+	OFONO_SIM_PHASE_1G,
+	OFONO_SIM_PHASE_2G,
+	OFONO_SIM_PHASE_2G_PLUS,
+	OFONO_SIM_PHASE_3G,
+	OFONO_SIM_PHASE_UNKNOWN,
+};
+
+enum ofono_sim_cphs_phase {
+	OFONO_SIM_CPHS_PHASE_NONE,
+	OFONO_SIM_CPHS_PHASE_1G,
+	OFONO_SIM_CPHS_PHASE_2G,
+};
+
 typedef void (*ofono_sim_file_info_cb_t)(const struct ofono_error *error,
 					int filelength,
 					enum ofono_sim_file_structure structure,
@@ -56,14 +90,22 @@ typedef void (*ofono_sim_imsi_cb_t)(const struct ofono_error *error,
 
 typedef void (*ofono_sim_ready_notify_cb_t)(void *data);
 
-typedef void (*ofono_sim_file_read_cb_t)(int ok,
-					enum ofono_sim_file_structure structure,
-					int total_length, int record,
+typedef void (*ofono_sim_file_read_cb_t)(int ok, int total_length, int record,
 					const unsigned char *data,
 					int record_length, void *userdata);
 
 typedef void (*ofono_sim_file_write_cb_t)(int ok, void *userdata);
 
+typedef void (*ofono_sim_passwd_cb_t)(const struct ofono_error *error,
+					enum ofono_sim_password_type type,
+					void *data);
+
+typedef void (*ofono_sim_lock_unlock_cb_t)(const struct ofono_error *error,
+					void *data);
+
+typedef void (*ofono_sim_locked_cb_t)(const struct ofono_error *error,
+					int locked, void *data);
+
 struct ofono_sim_driver {
 	const char *name;
 	int (*probe)(struct ofono_sim *sim, unsigned int vendor, void *data);
@@ -90,6 +132,26 @@ struct ofono_sim_driver {
 			ofono_sim_write_cb_t cb, void *data);
 	void (*read_imsi)(struct ofono_sim *sim,
 			ofono_sim_imsi_cb_t cb, void *data);
+	void (*query_passwd_state)(struct ofono_sim *sim,
+			ofono_sim_passwd_cb_t cb, void *data);
+	void (*send_passwd)(struct ofono_sim *sim, const char *passwd,
+			ofono_sim_lock_unlock_cb_t cb, void *data);
+	void (*reset_passwd)(struct ofono_sim *sim, const char *puk,
+			const char *passwd,
+			ofono_sim_lock_unlock_cb_t cb, void *data);
+	void (*change_passwd)(struct ofono_sim *sim,
+			enum ofono_sim_password_type type,
+			const char *old, const char *new,
+			ofono_sim_lock_unlock_cb_t cb, void *data);
+	void (*lock)(struct ofono_sim *sim, enum ofono_sim_password_type type,
+			int enable, const char *passwd,
+			ofono_sim_lock_unlock_cb_t cb, void *data);
+	void (*query_locked)(struct ofono_sim *sim,
+			enum ofono_sim_password_type type,
+			ofono_sim_locked_cb_t cb, void *data);
+	void (*envelope)(struct ofono_sim *sim, int length,
+				const guint8 *command,
+				ofono_sim_read_cb_t cb, void *data);
 };
 
 int ofono_sim_driver_register(const struct ofono_sim_driver *d);
@@ -106,6 +168,10 @@ void ofono_sim_set_data(struct ofono_sim *sim, void *data);
 void *ofono_sim_get_data(struct ofono_sim *sim);
 
 const char *ofono_sim_get_imsi(struct ofono_sim *sim);
+enum ofono_sim_phase ofono_sim_get_phase(struct ofono_sim *sim);
+
+enum ofono_sim_cphs_phase ofono_sim_get_cphs_phase(struct ofono_sim *sim);
+const unsigned char *ofono_sim_get_cphs_service_table(struct ofono_sim *sim);
 
 unsigned int ofono_sim_add_ready_watch(struct ofono_sim *sim,
 				ofono_sim_ready_notify_cb_t cb,
@@ -124,6 +190,7 @@ void ofono_sim_set_ready(struct ofono_sim *sim);
  * Returns 0 if the request could be queued, -1 otherwise.
  */
 int ofono_sim_read(struct ofono_sim *sim, int id,
+			enum ofono_sim_file_structure expected,
 			ofono_sim_file_read_cb_t cb, void *data);
 
 int ofono_sim_write(struct ofono_sim *sim, int id,
