@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -36,9 +36,13 @@
 #include "util.h"
 #include "smsutil.h"
 #include "simutil.h"
+#include "storage.h"
 
 #define CBS_MANAGER_INTERFACE "org.ofono.CbsManager"
 
+#define SETTINGS_STORE "cbs"
+#define SETTINGS_GROUP "Settings"
+
 static GSList *g_drivers = NULL;
 
 enum etws_topic_type {
@@ -69,6 +73,9 @@ struct ofono_cbs {
 	int ci;
 	char mnc[OFONO_MAX_MNC_LENGTH + 1];
 	char mcc[OFONO_MAX_MCC_LENGTH + 1];
+	ofono_bool_t powered;
+	GKeyFile *settings;
+	char *imsi;
 	const struct ofono_cbs_driver *driver;
 	void *driver_data;
 	struct ofono_atom *atom;
@@ -179,11 +186,19 @@ void ofono_cbs_notify(struct ofono_cbs *cbs, const unsigned char *pdu,
 	if (cbs->assembly == NULL)
 		return;
 
+	if (!cbs->powered)
+		return;
+
 	if (!cbs_decode(pdu, pdu_len, &c)) {
 		ofono_error("Unable to decode CBS PDU");
 		return;
 	}
 
+	if (cbs_topic_in_range(c.message_identifier, cbs->efcbmid_contents)) {
+		__ofono_cbs_sim_download(cbs->sim, pdu, pdu_len);
+		return;
+	}
+
 	if (!cbs_dcs_decode(c.dcs, &udhi, &cls, &charset, &comp, NULL, NULL)) {
 		ofono_error("Unknown / Reserved DCS.  Ignoring");
 		return;
@@ -267,6 +282,9 @@ static DBusMessage *cbs_get_properties(DBusConnection *conn,
 					OFONO_PROPERTIES_ARRAY_SIGNATURE,
 					&dict);
 
+	ofono_dbus_dict_append(&dict, "Powered", DBUS_TYPE_BOOLEAN,
+				&cbs->powered);
+
 	topics = cbs_topic_ranges_to_string(cbs->topics);
 	ofono_dbus_dict_append(&dict, "Topics", DBUS_TYPE_STRING, &topics);
 	g_free(topics);
@@ -276,6 +294,28 @@ static DBusMessage *cbs_get_properties(DBusConnection *conn,
 	return reply;
 }
 
+static char *cbs_topics_to_str(struct ofono_cbs *cbs, GSList *user_topics)
+{
+	GSList *topics = NULL;
+	char *topic_str;
+	struct cbs_topic_range etws_range = { 4352, 4356 };
+
+	topics = g_slist_append(topics, &etws_range);
+
+	if (user_topics != NULL)
+		topics = g_slist_concat(topics,
+					g_slist_copy(user_topics));
+
+	if (cbs->efcbmid_contents != NULL)
+		topics = g_slist_concat(topics,
+					g_slist_copy(cbs->efcbmid_contents));
+
+	topic_str = cbs_topic_ranges_to_string(topics);
+	g_slist_free(topics);
+
+	return topic_str;
+}
+
 static void cbs_set_topics_cb(const struct ofono_error *error, void *data)
 {
 	struct ofono_cbs *cbs = data;
@@ -308,6 +348,12 @@ static void cbs_set_topics_cb(const struct ofono_error *error, void *data)
 						CBS_MANAGER_INTERFACE,
 						"Topics",
 						DBUS_TYPE_STRING, &topics);
+
+	if (cbs->settings) {
+		g_key_file_set_string(cbs->settings, SETTINGS_GROUP,
+					"Topics", topics);
+		storage_sync(cbs->imsi, SETTINGS_STORE, cbs->settings);
+	}
 	g_free(topics);
 }
 
@@ -315,9 +361,8 @@ static DBusMessage *cbs_set_topics(struct ofono_cbs *cbs, const char *value,
 					DBusMessage *msg)
 {
 	GSList *topics;
-	GSList *etws_topics = NULL;
 	char *topic_str;
-	struct cbs_topic_range etws_range = { 4352, 4356 };
+	struct ofono_error error;
 
 	topics = cbs_extract_topic_ranges(value);
 
@@ -329,20 +374,108 @@ static DBusMessage *cbs_set_topics(struct ofono_cbs *cbs, const char *value,
 
 	cbs->new_topics = topics;
 
-	if (topics != NULL)
-		etws_topics = g_slist_copy(topics);
-
-	etws_topics = g_slist_append(etws_topics, &etws_range);
-	topic_str = cbs_topic_ranges_to_string(etws_topics);
-	g_slist_free(etws_topics);
-
 	cbs->pending = dbus_message_ref(msg);
+
+	if (!cbs->powered) {
+		error.type = OFONO_ERROR_TYPE_NO_ERROR;
+		cbs_set_topics_cb(&error, cbs);
+		return NULL;
+	}
+
+	topic_str = cbs_topics_to_str(cbs, topics);
 	cbs->driver->set_topics(cbs, topic_str, cbs_set_topics_cb, cbs);
 	g_free(topic_str);
 
 	return NULL;
 }
 
+static void cbs_set_powered_cb(const struct ofono_error *error, void *data)
+{
+	struct ofono_cbs *cbs = data;
+	const char *path = __ofono_atom_get_path(cbs->atom);
+	DBusConnection *conn = ofono_dbus_get_connection();
+	DBusMessage *reply;
+
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
+		ofono_debug("Setting Cell Broadcast topics failed");
+		if (!cbs->pending)
+			return;
+
+		__ofono_dbus_pending_reply(&cbs->pending,
+					__ofono_error_failed(cbs->pending));
+		return;
+	}
+
+	cbs->powered = !cbs->powered;
+
+	if (cbs->settings) {
+		g_key_file_set_boolean(cbs->settings, SETTINGS_GROUP,
+					"Powered", cbs->powered);
+		storage_sync(cbs->imsi, SETTINGS_STORE, cbs->settings);
+	}
+
+	ofono_dbus_signal_property_changed(conn, path,
+						CBS_MANAGER_INTERFACE,
+						"Powered",
+						DBUS_TYPE_BOOLEAN,
+						&cbs->powered);
+
+	if (!cbs->pending)
+		return;
+
+	reply = dbus_message_new_method_return(cbs->pending);
+	__ofono_dbus_pending_reply(&cbs->pending, reply);
+}
+
+static DBusMessage *cbs_set_powered(struct ofono_cbs *cbs, gboolean value,
+					DBusMessage *msg)
+{
+	const char *path = __ofono_atom_get_path(cbs->atom);
+	DBusConnection *conn = ofono_dbus_get_connection();
+	char *topic_str;
+
+	if (cbs->powered == value)
+		goto reply;
+
+	if (!cbs->driver->set_topics)
+		goto done;
+
+	if (msg)
+		cbs->pending = dbus_message_ref(msg);
+
+	if (value) {
+		topic_str = cbs_topics_to_str(cbs, cbs->topics);
+		cbs->driver->set_topics(cbs, topic_str,
+					cbs_set_powered_cb, cbs);
+		g_free(topic_str);
+	} else {
+		cbs->driver->clear_topics(cbs, cbs_set_powered_cb, cbs);
+	}
+
+	return NULL;
+
+done:
+	cbs->powered = value;
+
+	if (cbs->settings) {
+		g_key_file_set_boolean(cbs->settings, SETTINGS_GROUP,
+					"Powered", cbs->powered);
+		storage_sync(cbs->imsi, SETTINGS_STORE, cbs->settings);
+	}
+
+	ofono_dbus_signal_property_changed(conn, path,
+						CBS_MANAGER_INTERFACE,
+						"Powered",
+						DBUS_TYPE_BOOLEAN,
+						&cbs->powered);
+
+reply:
+	if (msg)
+		return dbus_message_new_method_return(msg);
+
+	return NULL;
+}
+
 static DBusMessage *cbs_set_property(DBusConnection *conn, DBusMessage *msg,
 					void *data)
 {
@@ -368,6 +501,17 @@ static DBusMessage *cbs_set_property(DBusConnection *conn, DBusMessage *msg,
 
 	dbus_message_iter_recurse(&iter, &var);
 
+	if (!strcmp(property, "Powered")) {
+		dbus_bool_t value;
+
+		if (dbus_message_iter_get_arg_type(&var) != DBUS_TYPE_BOOLEAN)
+			return __ofono_error_invalid_args(msg);
+
+		dbus_message_iter_get_basic(&var, &value);
+
+		return cbs_set_powered(cbs, value, msg);
+	}
+
 	if (!strcmp(property, "Topics")) {
 		const char *value;
 
@@ -437,6 +581,13 @@ static void cbs_unregister(struct ofono_atom *atom)
 		cbs->new_topics = NULL;
 	}
 
+	if (cbs->efcbmid_length) {
+		cbs->efcbmid_length = 0;
+		g_slist_foreach(cbs->efcbmid_contents, (GFunc)g_free, NULL);
+		g_slist_free(cbs->efcbmid_contents);
+		cbs->efcbmid_contents = NULL;
+	}
+
 	cbs->sim = NULL;
 
 	if (cbs->reset_source) {
@@ -447,6 +598,16 @@ static void cbs_unregister(struct ofono_atom *atom)
 			__ofono_netreg_set_base_station_name(cbs->netreg, NULL);
 	}
 
+	cbs->powered = FALSE;
+
+	if (cbs->settings) {
+		storage_close(cbs->imsi, SETTINGS_STORE, cbs->settings, TRUE);
+
+		g_free(cbs->imsi);
+		cbs->imsi = NULL;
+		cbs->settings = NULL;
+	}
+
 	if (cbs->netreg_watch) {
 		if (cbs->location_watch) {
 			__ofono_netreg_remove_status_watch(cbs->netreg,
@@ -514,6 +675,51 @@ struct ofono_cbs *ofono_cbs_create(struct ofono_modem *modem,
 	return cbs;
 }
 
+static void cbs_got_file_contents(struct ofono_cbs *cbs)
+{
+	gboolean powered;
+	GSList *initial_topics = NULL;
+	char *topics_str;
+	GError *error = NULL;
+
+	if (cbs->topics == NULL) {
+		if (cbs->efcbmi_contents != NULL)
+			initial_topics = g_slist_concat(initial_topics,
+					g_slist_copy(cbs->efcbmi_contents));
+
+		if (cbs->efcbmir_contents != NULL)
+			initial_topics = g_slist_concat(initial_topics,
+					g_slist_copy(cbs->efcbmir_contents));
+
+		cbs->topics = cbs_optimize_ranges(initial_topics);
+		g_slist_free(initial_topics);
+
+		topics_str = cbs_topic_ranges_to_string(cbs->topics);
+		g_key_file_set_string(cbs->settings, SETTINGS_GROUP,
+					"Topics", topics_str);
+		g_free(topics_str);
+		storage_sync(cbs->imsi, SETTINGS_STORE, cbs->settings);
+	}
+
+	if (cbs->efcbmi_length) {
+		cbs->efcbmi_length = 0;
+		g_slist_foreach(cbs->efcbmi_contents, (GFunc) g_free, NULL);
+		g_slist_free(cbs->efcbmi_contents);
+		cbs->efcbmi_contents = NULL;
+	}
+
+	if (cbs->efcbmir_length) {
+		cbs->efcbmir_length = 0;
+		g_slist_foreach(cbs->efcbmir_contents, (GFunc) g_free, NULL);
+		g_slist_free(cbs->efcbmir_contents);
+		cbs->efcbmir_contents = NULL;
+	}
+
+	powered = g_key_file_get_boolean(cbs->settings, SETTINGS_GROUP,
+						"Powered", &error);
+	cbs_set_powered(cbs, powered, NULL);
+}
+
 static void sim_cbmi_read_cb(int ok, int length, int record,
 				const unsigned char *data,
 				int record_length, void *userdata)
@@ -553,16 +759,9 @@ static void sim_cbmi_read_cb(int ok, int length, int record,
 	if (cbs->efcbmi_contents == NULL)
 		return;
 
-	cbs->efcbmi_contents = g_slist_reverse(cbs->efcbmi_contents);
-
 	str = cbs_topic_ranges_to_string(cbs->efcbmi_contents);
 	ofono_debug("Got cbmi: %s", str);
 	g_free(str);
-
-	cbs->efcbmi_length = 0;
-	g_slist_foreach(cbs->efcbmi_contents, (GFunc)g_free, NULL);
-	g_slist_free(cbs->efcbmi_contents);
-	cbs->efcbmi_contents = NULL;
 }
 
 static void sim_cbmir_read_cb(int ok, int length, int record,
@@ -607,16 +806,9 @@ static void sim_cbmir_read_cb(int ok, int length, int record,
 	if (cbs->efcbmir_contents == NULL)
 		return;
 
-	cbs->efcbmir_contents = g_slist_reverse(cbs->efcbmir_contents);
-
 	str = cbs_topic_ranges_to_string(cbs->efcbmir_contents);
 	ofono_debug("Got cbmir: %s", str);
 	g_free(str);
-
-	cbs->efcbmir_length = 0;
-	g_slist_foreach(cbs->efcbmir_contents, (GFunc)g_free, NULL);
-	g_slist_free(cbs->efcbmir_contents);
-	cbs->efcbmir_contents = NULL;
 }
 
 static void sim_cbmid_read_cb(int ok, int length, int record,
@@ -627,6 +819,7 @@ static void sim_cbmid_read_cb(int ok, int length, int record,
 	unsigned short mi;
 	int i;
 	char *str;
+	GSList *contents = NULL;
 
 	if (!ok)
 		return;
@@ -648,37 +841,56 @@ static void sim_cbmid_read_cb(int ok, int length, int record,
 		range->min = mi;
 		range->max = mi;
 
-		cbs->efcbmid_contents = g_slist_prepend(cbs->efcbmid_contents,
-							range);
+		contents = g_slist_prepend(contents, range);
 	}
 
-	if (cbs->efcbmid_contents == NULL)
+	if (contents == NULL)
 		return;
 
-	cbs->efcbmid_contents = g_slist_reverse(cbs->efcbmid_contents);
+	cbs->efcbmid_contents = g_slist_reverse(contents);
 
 	str = cbs_topic_ranges_to_string(cbs->efcbmid_contents);
 	ofono_debug("Got cbmid: %s", str);
 	g_free(str);
 
-	cbs->efcbmid_length = 0;
-	g_slist_foreach(cbs->efcbmid_contents, (GFunc)g_free, NULL);
-	g_slist_free(cbs->efcbmid_contents);
-	cbs->efcbmid_contents = NULL;
+	cbs_got_file_contents(cbs);
 }
 
 static void cbs_got_imsi(struct ofono_cbs *cbs)
 {
 	const char *imsi = ofono_sim_get_imsi(cbs->sim);
+	char *topics_str;
 
 	ofono_debug("Got IMSI: %s", imsi);
 
-	ofono_sim_read(cbs->sim, SIM_EFCBMI_FILEID,
-			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
-			sim_cbmi_read_cb, cbs);
-	ofono_sim_read(cbs->sim, SIM_EFCBMIR_FILEID,
-			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
-			sim_cbmir_read_cb, cbs);
+	cbs->settings = storage_open(imsi, SETTINGS_STORE);
+	if (cbs->settings == NULL)
+		return;
+
+	cbs->imsi = g_strdup(imsi);
+
+	cbs->topics = NULL;
+
+	topics_str = g_key_file_get_string(cbs->settings, SETTINGS_GROUP,
+						"Topics", NULL);
+	if (topics_str)
+		cbs->topics = cbs_extract_topic_ranges(topics_str);
+
+	/* If stored value is invalid or no stored value, bootstrap
+	 * topics list from SIM contents */
+	if (topics_str == NULL ||
+			(cbs->topics == NULL && topics_str[0] != '\0')) {
+		ofono_sim_read(cbs->sim, SIM_EFCBMI_FILEID,
+				OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+				sim_cbmi_read_cb, cbs);
+		ofono_sim_read(cbs->sim, SIM_EFCBMIR_FILEID,
+				OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
+				sim_cbmir_read_cb, cbs);
+	}
+
+	if (topics_str)
+		g_free(topics_str);
+
 	ofono_sim_read(cbs->sim, SIM_EFCBMID_FILEID,
 			OFONO_SIM_FILE_STRUCTURE_TRANSPARENT,
 			sim_cbmid_read_cb, cbs);
@@ -754,7 +966,7 @@ out:
 	 * that time
 	 */
 	if (lac_changed || ci_changed) {
-		cbs->reset_source = 
+		cbs->reset_source =
 			g_timeout_add_seconds(3, reset_base_station_name, cbs);
 	}
 
